/*
 * Created on 10.04.2006 for PIROL
 *
 * SVN header information:
 *  $Author: LBST-PF-3\orahn $
 *  $Rev: 2588 $
 *  $Date: 2006-10-24 10:36:51 +0200 (Di, 24 Okt 2006) $
 *  $Id: GaussKruegerProjection.java 2588 2006-10-24 08:36:51Z LBST-PF-3\orahn $
 */
package de.fhOsnabrueck.jump.pirol.utilities.projection.Projections;

import com.vividsolutions.jts.geom.Coordinate;

import de.fhOsnabrueck.jump.pirol.utilities.Properties.PropertiesHandler;
import de.fhOsnabrueck.jump.pirol.utilities.projection.ellipsoid.AbstractEllipsoid;
import de.fhOsnabrueck.jump.pirol.utilities.projection.ellipsoid.BesselEllipsoid;

/**
 * Descibes a Gauss-Krger projection depending on the Gauss-Krger zone.
 *
 * @author Ole Rahn
 * <br>
 * <br>FH Osnabr&uuml;ck - University of Applied Sciences Osnabr&uuml;ck,
 * <br>Project: PIROL (2006),
 * <br>Subproject: Daten- und Wissensmanagement
 * 
 * @version $Rev: 2588 $
 * 
 */
public class GaussKruegerProjection extends AbstractProjection {
    
    public final static String KEY_HELMERT_TRANSLATION_X = "translationX";
    public final static String KEY_HELMERT_TRANSLATION_Y = "translationY";
    public final static String KEY_HELMERT_TRANSLATION_Z = "translationZ";
    
    public final static String KEY_HELMERT_ROTATION_X = "rotationX";
    public final static String KEY_HELMERT_ROTATION_Y = "rotationY";
    public final static String KEY_HELMERT_ROTATION_Z = "rotationZ";
    
    public final static String KEY_HELMERT_MAPSCALE_FACTOR = "mapscaleFactor";
    
    // fill in default parameters (Labonde 1)
    protected double transX = -586,
                     transY = -87, 
                     transZ = -409,
                     rotX = 0.52,
                     rotY = 0.15,
                     rotZ = -2.82,
                     mapscale = 9.;
    
    // to WGS
    double mapscale_inv = 1./9.;
    
    protected double L0 = -1;
    protected double falseEasting = 0;
    
    protected int gaussKruegerZone = -1;
    
    /**
     * width of the meridian bands (3 or 6 degree), at this time only 3 degree is supported 
     */
    protected int bandWidth = 3;
    
    public final static int GAUSS_KRUEGER_BASE_SRID = 31464;
    public final static int GAUSS_KRUEGER_MAX_SRID = 31464 + 120;
    
    public GaussKruegerProjection(int gkZone) {
        super();
        this.gaussKruegerZone = gkZone;
        this.L0 = (this.gaussKruegerZone * this.bandWidth) * AbstractEllipsoid.degToRad;
        this.falseEasting = this.gaussKruegerZone * 1000000 + 500000;
        
        this.setEllipsoid(new BesselEllipsoid());
        this.fillInHelmertParametersFromProperties(null);
    }

    /**
     *@inheritDoc
     */
    public int getSRID() {
        if (this.bandWidth == 3)
            return GAUSS_KRUEGER_BASE_SRID + this.gaussKruegerZone;
        
        throw new IllegalStateException("don't know the ESRID if band are 6 degree wide");
    }
    
    
    
    public int getBandWidth() {
        return bandWidth;
    }

    public double getFalseEasting() {
        return falseEasting;
    }

    public int getGaussKruegerZone() {
        return gaussKruegerZone;
    }

    public double getL0() {
        return L0;
    }

    public double getMapscale() {
        return mapscale;
    }

    public double getMapscale_inv() {
        return mapscale_inv;
    }

    public double getRotX() {
        return rotX;
    }

    public double getRotY() {
        return rotY;
    }

    public double getRotZ() {
        return rotZ;
    }

    public double getTransX() {
        return transX;
    }

    public double getTransY() {
        return transY;
    }

    public double getTransZ() {
        return transZ;
    }

    public boolean fillInHelmertParametersFromProperties(PropertiesHandler helmertParameters){
        boolean allParametersThere = helmertParameters!=null && (
                helmertParameters.containsKey(KEY_HELMERT_TRANSLATION_X) && 
                helmertParameters.containsKey(KEY_HELMERT_TRANSLATION_Y) && 
                helmertParameters.containsKey(KEY_HELMERT_TRANSLATION_Z) && 
                helmertParameters.containsKey(KEY_HELMERT_ROTATION_X) && 
                helmertParameters.containsKey(KEY_HELMERT_ROTATION_Y) && 
                helmertParameters.containsKey(KEY_HELMERT_ROTATION_Z) && 
                helmertParameters.containsKey(KEY_HELMERT_MAPSCALE_FACTOR));

        if (allParametersThere){
            this.transX = helmertParameters.getPropertyAsDouble(KEY_HELMERT_TRANSLATION_X);
            this.transY = helmertParameters.getPropertyAsDouble(KEY_HELMERT_TRANSLATION_Y);
            this.transZ = helmertParameters.getPropertyAsDouble(KEY_HELMERT_TRANSLATION_Z);
            
            this.rotX = helmertParameters.getPropertyAsDouble(KEY_HELMERT_ROTATION_X);
            this.rotY = helmertParameters.getPropertyAsDouble(KEY_HELMERT_ROTATION_Y);
            this.rotZ = helmertParameters.getPropertyAsDouble(KEY_HELMERT_ROTATION_Z);
            
            this.mapscale = helmertParameters.getPropertyAsDouble(KEY_HELMERT_MAPSCALE_FACTOR);
        } else if (!allParametersThere && helmertParameters!=null){
            new IllegalArgumentException("the given parameter set is not complete - aborting");
        }
        
        if (Math.abs(this.rotX) > 0.01)
            this.rotX = this.rotX / 3600 * AbstractEllipsoid.degToRad;
        if (Math.abs(this.rotY) > 0.01)
            this.rotY = this.rotY / 3600 * AbstractEllipsoid.degToRad;
        if (Math.abs(this.rotZ) > 0.01)
            this.rotZ = this.rotZ / 3600 * AbstractEllipsoid.degToRad;
        
        if (Math.abs(this.mapscale) > 1)
            this.mapscale = 1.0 - (Math.abs(this.mapscale) / 1000000d);
        
        this.mapscale_inv = 1.0/this.mapscale;
        
        /*
        this.logger.printDebug("================");
        this.logger.printDebug("Using Helmert parameters:");
        this.logger.printDebug(KEY_HELMERT_TRANSLATION_X + ": " + this.transX);
        this.logger.printDebug(KEY_HELMERT_TRANSLATION_Y + ": " + this.transY);
        this.logger.printDebug(KEY_HELMERT_TRANSLATION_Z + ": " + this.transZ);
        this.logger.printDebug(KEY_HELMERT_ROTATION_X + ": " + this.rotX);
        this.logger.printDebug(KEY_HELMERT_ROTATION_Y + ": " + this.rotY);
        this.logger.printDebug(KEY_HELMERT_ROTATION_Z + ": " + this.rotZ);
        this.logger.printDebug(KEY_HELMERT_MAPSCALE_FACTOR + ": " + this.mapscale);
        this.logger.printDebug("================");
        */

        return true;
    }

    /**
     *@inheritDoc
     */
    String asString() {
        return AbstractProjection.KEY_GAUSS_KRUEGER;
    }
    
    public String toString() {
        return this.asString() + ", Zone " + this.getGaussKruegerZone();
    }

    /**
     * Method to read the GK zone out of a coordinate.
     *@param coord coordinate
     *@return gk zone of the coordinate
     */
    public final static int guessGkZone( Coordinate coord ){
        double x = coord.x/1000000;
        x = Math.floor(x);
        
        logger.printDebug("looks like zone " + x);
        if (x<0 || x>120){
        	// TODO: if this exception is thrown, ask user which GK zone he wants to use...
        	throw new IllegalArgumentException("coordinate does not at all look like a Gau-Krueger coordinate, please check: " + coord.toString() + " - would be zone " + x);
        }
        
        return (int)x;
    }
    
}
